import { reactive, watchEffect } from "vue";
import { glibevent, ctx } from "../../store";

export const dirtySpecs = reactive({});

export function useDirtyState() {
  const prompt = "Changes you made have not been saved. Are you sure?";

  const isDirty = () => {
    return ctx().isFormDirty && !ctx().isFormSubmitted && !confirm(prompt);
  };

  const clearDirtyState = () => {
    ctx().isFormDirty = false;
  };

  const setDirty = () => ctx().isFormDirty = true;

  const watchDirtyState = () => {
    return watchEffect(() => {
      if (ctx().isFormDirty) {
        window.onbeforeunload = () => !isDirty();
        glibevent.onbeforewindowsopen = () => !isDirty();
        glibevent.onbeforewindowsclose = () => !isDirty();
        glibevent.onbeforewindowsreload = () => !isDirty();
      } else {
        window.onbeforeunload = null;
        glibevent.onbeforewindowsopen = null;
        glibevent.onbeforewindowsclose = null;
        glibevent.onbeforewindowsreload = null;
      }
    });
  };

  return { clearDirtyState, setDirty, watchDirtyState, isDirty };
}